@@ -15,8 +15,8 @@
 #                                 #
 ###################################
 
-fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm",der=1,vfactor=15,iterlim=100,accuracy="standard",kmst=!dmm$st,kmrep=5,postst=TRUE) { 
-	mod=dmm #keep the original model
+fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="optim",vfactor=15,der=1,iterlim=100,kmst=!dmm$st,kmrep=5,postst=FALSE) { 
+	mod=dmm #keep copy of the original model
 ## create good starting values if not provdided
 	if(kmst) {
 		if(class(dmm)[1]!="dmm") {
@@ -61,7 +61,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 	}
 	
 	if(postst && !kmst) {
-		# boost provided startvalues
+		# boost provided startvalues using Viterbi algorithm
 		if(class(dmm)[1]=="dmm") {
 			logl=loglike(dat,dmm)$logl
 			pp=dmm$pars
@@ -75,13 +75,6 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 	
 	xgmod=checkSetRecode(dat,dmm,tdcov,printlevel)
 	
-## set bounds of shift parameter if 3weibull or 3gamma or 3lognormal
-	if(any(xgmod$itemtypes< -30)) print("to do: set bound of shift parameter of at least one but not all states on min (data)")
-	if(any(xgmod$itemtypes< 0)) {
-		der=0
-		warning("Can't compute derivatives for model with other than gaussian and multinomial items.")
-	}
-	
 	if(tdcov) {
 		if(der) {
 			der=0
@@ -93,7 +86,6 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		}
 	}
 	
-	
  	initLogl=loglike(dat=dat,dmm=xgmod,tdcov=tdcov,print=0,set=FALSE)$logl
  	if(is.nan(initLogl) || is.infinite(initLogl)) stop("Initial model inadmissible, better starting values needed?!\n")
  	if(printlevel>0) cat("Initial loglikelihood: ", initLogl, " \n")
@@ -101,8 +93,8 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 	timeUsed=0
 	
 	if(method=="npsol") {
-		if(!is.loaded(symbol.C("npsolc"))) {
-			method="nlm"
+		if(!is.loaded("npsolc")) {
+			method="donlp"
 			warning("Optimization method changed to nlm because npsol is not available on this computer.")
 		}
 	}
@@ -204,19 +196,19 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		} # else nothing to do, bl and bu are already defined
 		
 		## npsol optimization options
-		itlim=paste(c("Iteration Limit = ", iterlim , "   "), collapse=" ")
-		npopts=c(itlim,
-			"Print level = 20 ",
-			"Minor print level = 10",
-			"Derivative level = 0 ",
-			"Hessian = No ",
-			"Verify level = 0 ")
+ 		itlim=paste(c("Iteration Limit = ", iterlim , "   "), collapse=" ")
+ 		npopts=c(itlim,
+ 			"Print level = 20 ",
+ 			"Minor print level = 10",
+ 			"Derivative level = 0 ",
+ 			"Hessian = No ",
+ 			"Verify level = 0 ")
 		
 # 			"Optimality tolerance = 1.0e-12 ")
 		# verify options: 0 is good for standard markovian and latent class models
 		# verify level should be 3 for new types of models, ie with new distributions etc.
 		
-		optset=lapply(npopts,FUN=optfor <- function(opt) {.Fortran("npoptn",as.character(opt),PACKAGE="depmix") } )
+ 		optset=lapply(npopts,FUN=optfor <- function(opt) {.Fortran("npoptn",as.character(opt),PACKAGE="depmix") } )
 		# optfile can be used to read in an optionsfile called npoptn from the working directory
 		optfile=0 # using this will override above specified default options
 		derivatives=der # are derivatives to be used or not
@@ -263,12 +255,66 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		z
 	}
 	
-	if(method=="gloptim") {
+	##  call npmain to optimize the model (non-linear constraints not implemented yet)
+	if(method=="donlp") {
+			
+		bu=bu[which(fixed==1)]
+		bl=bl[which(fixed==1)]
 		
-		# information: fitpars, fixed, bu, bl, A, bulin, bllin (fixed is redundant ...)
-		# this is assuming that A has no redundant stuff anymore such as constraints
-		# containing fixed parameters etc. 
+		if(printlevel>29) {
+			print(A)
+			print(bl)
+			print(bu)
+			print(bllin)
+			print(bulin)
+		}
 		
+		# donlp specific inputs
+		optpars=fitpars[which(fixed==1)]
+		A=A[,which(fixed==1),drop=FALSE]			
+		
+		# define loglike function
+		logl <- function(pars) {
+			xgmod$pars[which(fixed==1)]=pars
+			-loglike(dat=dat,dmm=xgmod,print=0,set=FALSE,tdcov=tdcov)$logl
+		}
+		
+		if(der) {
+			grad <- function(pars) {
+				xgmod$pars[which(fixed==1)]=pars
+				gr <- -loglike(dat=dat,dmm=xgmod,print=0,set=FALSE,tdcov=tdcov,grad=TRUE)$gr[which(fixed==1)]
+				return(gr)
+			}
+			attr(logl, "gr") <- grad
+		}
+		
+		timeUsed <- system.time(
+			res <- donlp2(optpars, logl,
+				par.upper=bu,
+				par.lower=bl,
+				A = A,
+				lin.upper=bulin,
+				lin.lower=bllin,
+				nlin = list(),
+				control=donlp2.control(),
+				env=.GlobalEnv, name="Rdonlp2")
+		)
+		
+		z=list()
+		z$objf=-res$fx
+		z$iter=res$step.nr
+		z$inform=res$message
+		z$timeUsed=timeUsed[3]
+		fitpars[which(fixed==1)]=res$par
+		z$pars=fitpars
+		z$npars=xgmod$npars
+		z$totMem=NULL
+		z
+	}
+	
+	##  call optim or nlm to optimize the model
+	if(method=="optim" || method=="nlm") {
+		vfactor=15
 		# reparametrize for optim and nlm
 		noptpars=length(fitpars)
 		# un-constrained pars
@@ -280,13 +326,12 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 				else constr[i]=1
 			}
 		}
-		
 		# remove cols corresponding with non-constrained and fixed pars
 		A=A[,which(constr==1),drop=FALSE]
-		
 		# remove rows correponding with inequality constraints
 		idx=which(bllin<bulin) 
 		ci=matrix(bulin,nc=1) 
+		
 		ineq=FALSE
 		if(length(idx)>0) {
 			A=A[-idx,,drop=FALSE]
@@ -315,150 +360,6 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			pars=ucpars
 		}
 		
-		# bounds on original parameters = linear constraints on new parameters
-		bucon=bu[which(constr==1)]
-		blcon=bl[which(constr==1)]
-		uiOpt=rbind(-b,b)
-		ciOpt=c((-bucon+ginvA%*%ci),(blcon-ginvA%*%ci))
-		
-		if(nucpars>0) {
-			uiOpt=bdiag(list(uiOpt,diag(rep(0,nucpars))))
-			ciOpt=c(ciOpt,rep(-1e-10,nucpars))
-		}
-		
-		# linear inequality constraints on original parameters
-		if(ineq) {
-			uiOptLin=rbind(-Aineq%*%b,Aineq%*%b)
-			if(nucpars>0) {
-				zz=matrix(0,nrow(uiOptLin),nucpars)
-				uiOptLin=cbind(uiOptLin,zz)
-			}
-			ciOpt=c(ciOpt,Aineq%*%ginvA%*%ci-ciIneqUp,-Aineq%*%ginvA%*%ci+ciIneqLow)
-			uiOpt=rbind(uiOpt,uiOptLin)
-		}
-				
-		assign("fcalls",0,envir=.GlobalEnv)
-		
-		# gloptim optimization
-		f <- function(pars) {
-			# expand pars
-			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
-			if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
-			ll=loglike(dat=dat,dmm=xgmod,print=0,set=FALSE,tdcov=tdcov)$logl
-			xgpars=xgmod$pars
-			assign("fcalls",fcalls+1,envir=.GlobalEnv)
-			if(is.nan(ll) || is.infinite(ll)) ll=initLogl*1.01
-			pr=0
-			if(printlevel>9 && (fcalls%%10)==0) {cat("call " , fcalls, " logl ", ll, "\n"); pr=1}
-			if(printlevel>14 && pr==0) cat("call " , fcalls, " logl ", ll, "\n")
-			ll
-		}
-		
-		gr <- function(pars) {
-			# expand pars
-			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
-			if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
-			gr=loglike(dat=dat,dmm=xgmod,grad=TRUE,print=0,set=FALSE)$gr
-			if(nnewpars>0) grad=c(gr[which(constr==1)]%*%b,gr[which(uncon==1)])
-			else grad=gr[which(uncon==1)]
-			if(printlevel>0 && (fcalls%%10)==0) cat("call " , fcalls, " grad ", grad, "\n")
-			if(printlevel>9) cat("call " , fcalls, " grad ", grad, "\n")
-			grad
-		}
-		
-		# optim control parameters
-		nopt=1
-		fnscale=-1.0
-		trace=1
-		factr=1e7
-		maxit=5000
-		REPORT=10
-		if(printlevel>1 && printlevel<5) REPORT=5
-		if(printlevel>4) REPORT=1
-		if(printlevel==0) REPORT=0
-		control=list(fnscale=fnscale,trace=trace,factr=factr,maxit=maxit,REPORT=REPORT)
-		oldObjf=initLogl
-		
-		for(i in 1:nopt) { 
-			# this actually seems to be working just fine, but it does depend on mu a little bit
-			
-			if(der) z<-constrOptim(theta=pars,f,gr=gr,ui=uiOpt,ci=ciOpt,control=control,outer.iterations=500,mu=1e-04)
-			if(der==0) z<-constrOptim(theta=pars,f,gr=NULL,ui=uiOpt,ci=ciOpt,control=control,outer.iterations=500,mu=5e-01)
-			print(z)
-			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
-			if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
-			
-			if(totalviolation > 1e-9) {
-				cat("Constraint violation not zero, minimum probably not optimal, provide better starting values.\n")
-				cat("Violation is: ", totalviolation, "\n")
-			}
-			
-			if(z$convergence==0 && totalviolation < 1e-9) break
-			violationPenalty=3*violationPenalty
-			oldObjf=z$value
-			pars=z$par
-		}
-		names(z)=c("par","objf","iter","inform","message")
-		
-		# put fitted pars into xgmod, gather output into z
-		if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
-		if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
-		
-		z$timeUsed=timeUsed
-		z$pars=xgmod$pars
-		z$npars=xgmod$npars
-		z$totMem=NULL
-		
-	} # end gloptim
-	
-##  call optim or nlm to optimize the model
-	if(method=="optim" || method=="nlm") {
- 	
-	# reparametrize for optim and nlm
-		noptpars=length(fitpars)
-		# un-constrained pars
-		uncon=rep(0,noptpars)
-		constr=rep(0,noptpars)
-		for(i in 1:noptpars) {
-			if(fixed[i]==1) {
-				if(sum(as.logical(xgmod$A[,i]))==0) uncon[i]=1
-				else constr[i]=1
-			}
-		}
-		# remove cols corresponding with non-constrained and fixed pars
- 		A=A[,which(constr==1),drop=FALSE]
-		# remove rows correponding with inequality constraints
- 		idx=which(bllin<bulin) 
- 		ci=matrix(bulin,nc=1) 
-		
-		ineq=FALSE
-  		if(length(idx)>0) {
- 			A=A[-idx,,drop=FALSE]
- 			ci=ci[-idx]
-			Aineq=xgmod$A[idx,which(constr==1),drop=FALSE]
-			ciIneqLow=matrix(xgmod$bllin[idx],nc=1)
-			ciIneqUp=matrix(xgmod$bulin[idx],nc=1)
-			ineq=TRUE
-		}
-		
-		if(nrow(A)>0) {
-			b=Null(t(A))
-			ginvb=ginv(b)
-			ginvA=ginv(A)
-			newpars=ginvb%*%(fitpars[which(constr==1)]-ginvA%*%ci)
-			nnewpars=length(newpars)
-		} else {
-			nnewpars=0
-		}
-		
-		ucpars=xgmod$pars[which(uncon==1)]
-		nucpars=length(ucpars)
-		if(nnewpars>0) {
-			pars=c(newpars,ucpars)
-		} else {
-			pars=ucpars
-		}
-		
 		violationPenalty=vfactor
 		
 		viol <- function(xgpars) {
@@ -475,11 +376,11 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			violation=violcon+violup+viollow+sum(violineq)
 			violation
 		}
-				
+		
 		assign("fcalls",0,envir=.GlobalEnv)
 		assign("totalviolation",0,envir=.GlobalEnv)		
 		
-	# optim optimization
+		# optim optimization
 		if(method=="optim") {
 			f <- function(pars) {
 				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
@@ -508,7 +409,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 				grad
 			}
 			
-	# optim control parameters
+			# optim control parameters
 			nopt=1
 			fnscale=-1.0
 			trace=1
@@ -522,7 +423,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			oldObjf=initLogl
 			
 			for(i in 1:nopt) { 
-  				timeUsed=timeUsed+system.time(z<-optim(pars,f,gr=NULL,control=control,method="BFGS",hessian=FALSE))[3]
+				timeUsed=timeUsed+system.time(z<-optim(pars,f,gr=NULL,control=control,method="BFGS",hessian=FALSE))[3]
 				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
 				if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
 				
@@ -539,7 +440,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			}
 			names(z)=c("par","objf","iter","inform","message")
 		} # end optim
-
+		
 		# nlm optimization
 		if(method=="nlm") {
 			assign("loglvalue",0,envir=.GlobalEnv)
@@ -571,8 +472,6 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			if(printlevel==0) prl=0
 			if(printlevel>4) prl=2
 			ndig=12
-			if(accuracy=="high") ndig=15
-			if(accuracy=="best") ndig=18
 			nopt=1
 			adddig=3
 			
@@ -597,10 +496,10 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			z$objf=ifelse(loglvalue==0,-z$objf,loglvalue)
 		} # end nlm
 		
-	# put fitted pars into xgmod, gather output into z
+		# put fitted pars into xgmod, gather output into z
 		if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
 		if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
-		
+			
 		z$timeUsed=timeUsed
 		z$pars=xgmod$pars
 		z$npars=xgmod$npars
@@ -879,7 +778,7 @@ computeSes <- function(dat,dmm) {
 
 # compute bootstrap based standard errors + gof
 bootstrap <- function(object, dat, samples=100, pvalonly=0, ...) {
-	if(!class(object)=="fit") error("Bootstrapping only possible on 'fit' objects.")
+	if(!class(object)=="fit") stop("Bootstrapping only possible on 'fit' objects.")
 	mod=object$mod
 	# compute only bootrapped pval, not ses
 	gof=matrix(0,samples,3)
@@ -892,11 +791,9 @@ bootstrap <- function(object, dat, samples=100, pvalonly=0, ...) {
 			PACKAGE="depmix")
 	}
 	for(i in 1:samples) {
-		
 		gen=generate(dmm=mod, nt=dat$ntimes)
-		
 		ll=loglike(dat=gen,dmm=mod,print=0)$logl
-		fit=fitdmm(dat=gen,dmm=mod,print=0,ses=0,post=0)
+		fit=fitdmm(dat=gen,dmm=mod,print=0,ses=0,postst=0)
 		bootpars[i,]=fit$mod$pars
 		ll=fit$loglike
 		gof[i,]=c(ll,fit$aic,fit$bic)
